/* Pour getline() */
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <ctype.h>

#ifdef _EFENCE
#include <efence.h>
#endif

#include <MlvProcC.h>
#include <MlvTypeC.h>

#define ECRAN_LARGEUR 350
#define ECRAN_HAUTEUR 400
#define POLICE "-misc-*-*-*-*-*-*-*-*-*-*-*-*-*"

#define DELAI 5

/* Types */

typedef struct avion {
    char identifiant[6];
    int carburant;
    int consommation;
    char heure_decollage[4];
    struct compagnie *compagnie;
} Avion;

typedef struct cellule_avion {
    Avion avion;
    struct cellule_avion *suivant_compagnie;
    struct cellule_avion *precedent_compagnie;
    struct cellule_avion *suivant_attente;
} Cellule_avion, *Liste_avions;

typedef struct compagnie {
    char *nom;
    char acronyme[3];
    Liste_avions avions_compagnie;
} Compagnie;

typedef struct cellule_compagnie {
    Compagnie comp;
    struct cellule_compagnie *suivant;
} Cellule_compagnie, *Liste_compagnie;

typedef struct queue {
    Liste_avions premier;
    Liste_avions dernier;
} Queue;

typedef struct histo {
    char txt[19];
    struct histo * suivant;
} Historique;

typedef struct noire {
    Compagnie * comp;
    struct noire * suiv;
} Cellule_noire, *Liste_noire;

typedef enum { FAUX, VRAI } bool;

typedef enum {
    NERIENFAIRE,
    AJOUT,
    SUPPRIMER,
    URGENCE,
    AJOUTCOMPAGNIE,
    LISTENOIRE,
    ALEATOIRE,
    CHARGER,
    STATUS,
    HISTORIQUE,
    QUITTER
} Option;

typedef enum { D, A, U, N, C } Action;

/* Prototypes */

  /* aeroport.c */
bool evenement_utilisateur(Option bouton, Liste_compagnie * comp,
	Liste_avions * atterrissages, Liste_avions * decollages, 
	Historique * histo, Queue * queue, Liste_noire * noire, int t);
void evenement_aleatoire(Liste_compagnie * comp, Liste_avions * atte,
	Liste_avions * decol, Historique * histo, Queue * queue, 
	Liste_noire * noire, int t);
bool options(Liste_compagnie * comp, int * nombre_pistes, Liste_avions *deco,
	Liste_avions *atte, Liste_noire noire, int *t, int argc, char **argv);

  /* affichage.c */
void dessin_bouton(char *texte, int p, MlvStruct * ecran);
void affichage_boutons(MlvStruct * ecran);
void affichage_atterrissages(Liste_avions atterrissages, MlvStruct * ecran);
void affichage_decollages(Liste_avions decollages, Queue * queue,
	MlvStruct * ecran);
void affichage_historique(Historique * historique, MlvStruct * ecran);
void affichage(MlvStruct * ecran, int t, Liste_avions decollages,
	Liste_avions atterrissages, Queue * queue, Historique * historique,
	int nombre_pistes);

  /* memoire.c */
bool alloue_compagnie(Liste_compagnie * comp, char * nom, char acro[3]);
bool alloue_avion(Liste_avions *avions, char id[6], int carburant, 
	int conso, char heure_decollage[4], struct compagnie * compagnie);
void ajouter_decollage(Liste_avions * decollages, Liste_avions avion, int t);
void ajouter_atterrissage(Liste_avions * atterrissages, Liste_avions avion);
bool historique_ajouter(Historique ** historique, char *txt);
bool mettre_sur_liste_noire(Liste_noire * noire, Compagnie * comp);
void liberer_compagnies(Liste_compagnie * comp);
void detruire_avion(Liste_avions avion);
void liberer_avions(Liste_avions * avions);
void effacer_historique(Historique ** historique);
void effacer_liste_noire(Liste_noire * noire);

  /* es.c */
Option bouton(MlvStruct * ecran);
void ecrire_nchar(char *txt, int n);
Liste_compagnie demande_compagnie(Liste_compagnie * comp, char acro[3]);
void demande_avion(Liste_compagnie * comp, Liste_avions * decoll,
	Liste_avions * atterrissage, Liste_noire noire, int t);
void status(Liste_compagnie comp, Liste_noire noire, int t);
void supprimer(Liste_avions * decollage, Queue * queue);
void charger_fichier(Liste_compagnie *comp, Liste_avions *decollages,
	Liste_avions *atterrissages, Liste_noire noire, int t, char *nom_fichier);
void charger_compagnies(char *nom_fichier, Liste_compagnie * comp);
void lire_historique(Historique * histo);
void mettre_en_urgence(Liste_avions atterrissages);
void liste_noire(Liste_compagnie compagnies, Liste_avions * decollages,
	Liste_avions atterrissages, Liste_noire * noire);
void historique(Historique ** historique, Action action, Liste_avions avion,
	int t);

  /* gestion.c */
void evenements(Liste_compagnie * comp, Liste_avions * atterrissages, 
	Liste_avions * decollages, Queue * queue, int t,
	Historique ** historique, Liste_noire * noire, int n_pistes);
void gestion_carburant(Liste_avions atterrissages);
bool atterrissage(Liste_avions * atterrissages, Historique ** histo);
void crashs(Liste_avions * atterrissages, Historique ** histo, int t);
bool decollage(Liste_avions * decollages, Queue * queue, int t, 
	Historique ** histo);
void gestion_piste(Liste_avions decollages, Queue * queue, int t);
void detection_urgences(Liste_avions atterrissages);
bool atterrissage_urgence(Liste_avions * atterrissages, Historique ** histo);
bool atterrissage_noire(Liste_avions * atterrissages, Historique ** histo,
	Liste_noire noire);

  /* annexe.c */
bool hvalide(char heure[4]);
bool identifiant_valide(char *id);
bool acronyme_valide(char *acro);
Liste_avions avion_connu(Liste_avions avion, char id[6]);
Liste_compagnie comp_connue(Liste_compagnie comp, char acro[3]);
bool sur_liste_noire(Liste_noire noire, Compagnie * comp);
int piste_libre(bool pistes[], int n_pistes);
int vollibre(Liste_avions avions);
int horaire_minute(char heure[4]);
int vollibre(Liste_avions avions);
char * minute_horaire(int t);
char * chaine_avion(Liste_avions avion, bool decollage);
char * affichage_heure(int heure);
unsigned int aleatoire(unsigned int a, unsigned int b);
