/* affichage.
 * Gestion de l'affichage graphique du programme
 */

#include "../include/aeroport.h"

/*
 * Dessine un bouton dans la zone des boutons  la position p
 * ( en partant du bas )
 */
void dessin_bouton(char *texte, int p, MlvStruct *ecran)
{
    if ( strlen(texte) > 19 )
	fprintf(stderr,"Attention, texte trop long\n");
    p *= 15;
    draw_filled_rectangle(ECRAN_LARGEUR-149,ECRAN_HAUTEUR-p-14,148,13,
	    "gray90",ecran);
    draw_line(ECRAN_LARGEUR-150,ECRAN_HAUTEUR-p,ECRAN_LARGEUR,
	    ECRAN_HAUTEUR-p,"black",ecran);
    draw_string(ECRAN_LARGEUR-140,ECRAN_HAUTEUR-p-3,texte,
	    POLICE,"black",ecran);
}

/*      affichage_boutons
 * Dessine les boutons  l'cran
 */
void affichage_boutons(MlvStruct *ecran)
{
    dessin_bouton("Continuer",10,ecran);
    dessin_bouton("Ajouter un avion",9,ecran);
    dessin_bouton("Charger un fichier",8,ecran);
    dessin_bouton("Supprimer un avion",7,ecran);
    dessin_bouton("Avion en urgence !",6,ecran);
    dessin_bouton("Ajout compagnie",5,ecran);
    dessin_bouton("Ajout  Liste Noire",4,ecran);
    dessin_bouton("vnement alatoire",3,ecran);
    dessin_bouton("Status",2,ecran);
    dessin_bouton("Historique",1,ecran);
    dessin_bouton("Quitter",0,ecran);
}

/*** affichage_atterrissages
 * Affiche les 12 prochains avions  l'attrissage
 */
void affichage_atterrissages(Liste_avions atterrissages, MlvStruct * ecran)
{
    char *txt;
    int compte = 1;

    draw_string(5,13,"Prochains attrissages",POLICE,"red",ecran);
    while ( atterrissages != NULL && compte < 13 ) {
	txt = chaine_avion(atterrissages,FAUX);
	draw_string(15,15+compte*15,txt,POLICE,"black",ecran);
	atterrissages = atterrissages->suivant_attente;
	free(txt);
	compte++;
    }
}

/*** affichage_decollages
 * Affiche les 12 prochains avions au dcollage
 * Un message spare les avions sur la piste de ceux encore au parking, si tel 
 * est le cas, il n'y a que 11 avions affichs.
 */
void affichage_decollages(Liste_avions decollages, Queue * queue,
	MlvStruct * ecran)
{
    char *txt;
    int compte = 0;
    bool enpiste = FAUX;
    draw_string(5,213,"Avions en bout de piste",POLICE,"red",ecran);

    if ( decollages == queue->premier )
	enpiste = VRAI;

    while ( decollages != NULL && compte < 12 ) {
	if ( !enpiste ) {
	    draw_string(5,230+compte*15,"Prochains dcollages",POLICE,
		    "red",ecran);
	    compte++;
	    enpiste = VRAI;
	}

	if ( decollages == queue->dernier )
	    enpiste = FAUX;

	txt = chaine_avion(decollages,VRAI);
	draw_string(15,230+compte*15,txt,POLICE,"black",ecran);
	free(txt);
	
	decollages = decollages->suivant_attente;
	compte++;
    }
}

/*** affichage_historique
 * Affiche les 3 derniers vnements de l'historique
 */
void affichage_historique(Historique * historique, MlvStruct * ecran)
{
    int compte = 0;

    draw_string(ECRAN_LARGEUR-143,45,"Historique",POLICE,"red",ecran);

    while ( historique != NULL && compte < 12 ) {
	draw_string(ECRAN_LARGEUR-140,60+15*compte,historique->txt,
		POLICE,"blue",ecran);
	historique = historique->suivant;
	compte++;
    }
}

/*** affichage
 * Dessin gnral de l'interface et appel des fonctions d'affichage
 */
void affichage(MlvStruct * ecran, int t, Liste_avions decollages,
	Liste_avions atterrissages, Queue * queue, Historique * historique,
	int nombre_pistes)
{
    char horloge[6], *heure, msg[128];
    /* Nettoyage */
    draw_filled_rectangle(0,0,ECRAN_LARGEUR,ECRAN_HAUTEUR,"white",ecran);
    
    /* Dlimitations */
      /* verticale */
    draw_line(ECRAN_LARGEUR-150,0,ECRAN_LARGEUR-150,ECRAN_HAUTEUR,"black",ecran);
      /* entre arrives et dparts */
    draw_line(0,ECRAN_HAUTEUR/2,ECRAN_LARGEUR-150,ECRAN_HAUTEUR/2,"black",ecran);
      /* pour l'horloge */
    draw_line(ECRAN_LARGEUR-150,30,ECRAN_LARGEUR,30,"black",ecran);
      /* pour les boutons */
    draw_line(ECRAN_LARGEUR-150,ECRAN_HAUTEUR-165,ECRAN_LARGEUR,
	    ECRAN_HAUTEUR-165,"black",ecran);

    /* Horloge */
    if ( (heure = affichage_heure(t)) != NULL ) {
	draw_filled_rectangle(ECRAN_LARGEUR-149,0,148,29,"gray90",ecran);
	sprintf(horloge,"%c%c:%c%c",heure[0],heure[1],heure[2],heure[3]);
	draw_string(ECRAN_LARGEUR-93,13,horloge,POLICE,"black",ecran);
	free(heure);
    } else {
	draw_string(ECRAN_LARGEUR-93,13,"ERREUR",POLICE,"black",ecran);
    }

    /* Nombre de pistes */
    sprintf(msg,"Nombre de pistes : %d",nombre_pistes);
    draw_string(ECRAN_LARGEUR-145,27,msg,POLICE,"black",ecran);

    /* Boutons */
    affichage_boutons(ecran);

    /* Prochains dcollages/attrissages */
    affichage_atterrissages(atterrissages,ecran);
    affichage_decollages(decollages,queue,ecran);

    /* Historique */
    affichage_historique(historique,ecran);

    /* Affichage */
    display_window(ecran);
}
