/* annexe.c
 * Fonctions annexes.
 * Vrifier des donnes, obtenir des informations, convertir des informations
 */

#include "../include/aeroport.h"

/*** hvalide
 * Retourne VRAI si heure est un horaire valide au format hhmm
 */
bool hvalide(char heure[4])
{
    if ( heure[0] < '0' || heure[0] > '2' )
	return FAUX;
    if ( heure[1] < '0' || ( heure[1] > '3' && heure[0] == '2' )
	    || heure[1] > '9' )
	return FAUX;
    if ( heure[2] < '0' || heure[2] > '5' )
	return FAUX;
    if ( heure[3] < '0' || heure[3] > '9' )
	return FAUX;
    return VRAI;
}

/*** identifiant_valide
 * Retourne VRAI si l'identifiant est valide, 3 caractres majuscules
 * suivi de 3 chiffres
 */
bool identifiant_valide(char *id)
{
    int i;

    for ( i=0 ; i < 3 ; i++ )
	if ( ! isupper(id[i]) )
	    return FAUX;
    for ( i=3 ; i < 6 ; i++ )
	if ( ! isdigit(id[i]) )
	    return FAUX;

    return VRAI;
}

/*** acronyme_valide
 * Retourne VRAI si l'acronyme est bien compos de 3 caractres majuscules 
 */
bool acronyme_valide(char *acro)
{
    int i;
    
    for ( i=0 ; i < 3 ; i++ )
	if ( ! isupper(acro[i]) )
	    return FAUX;

    return VRAI;
}

/*** avion_connu
 * Retourne l'adresse de l'avion s'il est connu de son identifiant
 */
Liste_avions avion_connu(Liste_avions avion, char id[6])
{
    while ( avion != NULL ) {
	if ( memcmp(avion->avion.identifiant,id,6) == 0 )
	    return avion;
	avion = avion->suivant_compagnie;
    }

    return NULL;
}

/*** comp_connue
 * Retourne vrai si l'acronyme pass en paramtre est celui d'une
 * compagnie existante
 */
Liste_compagnie comp_connue(Liste_compagnie comp, char acro[3])
{
    while ( comp != NULL ) {
	if ( memcmp(comp->comp.acronyme,acro,3) == 0 )
	    return comp;
	comp = comp->suivant;
    }
    return NULL;
}

/*** sur_liste_noire
 * Retourne VRAI si la compagnie passe en argument est sur liste noire
 */
bool sur_liste_noire(Liste_noire noire, Compagnie * comp)
{
    while ( noire != NULL ) {
	if ( noire->comp == comp )
	    return VRAI;
	noire = noire->suiv;
    }
    return FAUX;
}

/*** piste_libre
 * Retourne un numro de piste libre. -1 est retourn si plus aucune piste 
 * n'est libre
 */
int piste_libre(bool pistes[], int n_pistes)
{
    int i;

    for ( i=0 ; i < n_pistes ; i++ )
	if ( pistes[i] == FAUX )
	    return i;

    return -1;
}

/*** vollibre
 * Retourne un numro de vol libre dans la liste des avions
 */
int vollibre(Liste_avions avions)
{
    if ( avions == NULL )
	return aleatoire(1,999);

    int vol, limite;
    char id[6];
    memcpy(id,avions->avion.compagnie->acronyme,3);

    vol = aleatoire(1,999);
    limite = vol - 1;
    while ( vol != limite ) {
	id[3] = (vol / 100) + '0';
	id[4] = ((vol - (id[3]-'0') * 100) / 10) + '0';
	id[5] = (vol - (id[3]-'0') * 100 - (id[4]-'0') * 10) + '0';
	if ( avion_connu(avions,id) == NULL )
	    return vol;
	vol = ( vol < 1000 ) ? vol+1 : 1 ;
    }

    return -1;
}

/*** horaire_minute
 * Retourne un entier correspondant au nombre de minutes dans l'horaire prcis
 * ex : 2359 ( 23h59 ) -> 1439 minutes
 */
int horaire_minute(char heure[4])
{
    return (heure[3]-'0') + (heure[2]-'0')*10 
	+ (heure[1]-'0')*60 + (heure[0]-'0')*600;
}

/*** minute_horaire
 * Retourne une chaine de caractres ( alloue avec malloc() ) contenant
 * l'horaire au format hhmm correspondant au temps pass en argument
 */
char * minute_horaire(int t)
{
    char *heure;
    int tmp;

    if ( (heure = malloc(4)) == NULL )
	return NULL;

    tmp = t/600;
    heure[0] = tmp + '0';
    t -= tmp * 600;

    tmp = t/60;
    heure[1] = tmp + '0';
    t -= tmp * 60;

    tmp = t/10;
    heure[2] = tmp + '0';
    t -= tmp * 10;

    heure[3] = tmp + '0';

    return heure;
}

/*** chaine_avion
 * Retourne une chaine de caractres dans le format de codage des vnements
 * correspondant  l'avion pass en paramtre, avec un caractre '\0' final.
 * Cette chaine devra tre libre  l'aide de free().
 */
char * chaine_avion(Liste_avions avion, bool decollage)
{
    char * chaine = NULL;

    if ( (chaine = (char *) malloc(19 * sizeof(char))) == NULL )
	return NULL;

    switch(decollage) {
	case VRAI:
	    memcpy(chaine,avion->avion.identifiant,6);
	    chaine[6] = chaine[8] = '-';
	    chaine[7] = 'D';
	    memcpy(&chaine[9],avion->avion.heure_decollage,4);
	    sprintf(&chaine[13],"------");
	    break;
	case FAUX:
	    memcpy(chaine,avion->avion.identifiant,6);
	    chaine[6] = '-';
	    chaine[7] = 'A';
	    sprintf(&chaine[8],"------");
	    sprintf(&chaine[14],"%02d",avion->avion.carburant);
	    chaine[16] = '-';
	    sprintf(&chaine[17],"%02d",avion->avion.consommation);
	    break;
    }

    return chaine;
}

/*      affichage_heure
 * Convertit l'entier fournit ( reprsentant des minutes ) en chaine de 
 * caractres de la forme : hhmm
 */
char * affichage_heure(int heure)
{
    char * temps;
    if ( (temps = malloc(6)) == NULL )
	return NULL;
    sprintf(temps,"%02d%02d",heure/60,heure%60);
    return temps;
}

/*** aleatoire
 * Retourne un entier alatoire entre a et b en utilisant le gnrateur alatoire
 * de donnes de Linux, /dev/urandom
 */
unsigned int aleatoire(unsigned int a, unsigned int b)
{
    FILE *urandom;
    unsigned int r;

    /* On change a et b si b est plus petit que a */
    if ( b < a ) {
	r = b;
	b = a;
	a = r;
    }

    if ( (urandom = fopen("/dev/urandom","r")) == NULL ) {
	fprintf(stderr,"Erreur : impossible de gnrer un nombre alatoire");
	return b;
    }

    /* Lecture d'un entier dans /dev/urandom */
    fread(&r,sizeof(int),1,urandom);
    fclose(urandom);

    /* Ajustement de l'entier pour tre dans les bornes spcifies */
    r = r%(b-a+1) + a;

    return r;
}
