/* es.c
 * Gestion des entres/sorties du programme
 */

#include "../include/aeroport.h"

/*** bouton
 * Retourne un des lments de Option en fonction de la position du clic
 * sur l'cran
 */
Option bouton(MlvStruct * ecran)
{
    int x,y;
    
    do mouse_wait(&x,&y,ecran);
	while ( x < ECRAN_LARGEUR-150 || y < ECRAN_HAUTEUR-165 );
    
    if ( y < ECRAN_HAUTEUR-150 )
        return NERIENFAIRE;
    if ( y < ECRAN_HAUTEUR-135 )
        return AJOUT;
    if ( y < ECRAN_HAUTEUR-120 )
        return CHARGER;
    if ( y < ECRAN_HAUTEUR-105 )
        return SUPPRIMER;
    if ( y < ECRAN_HAUTEUR-90 )
        return URGENCE;
    if ( y < ECRAN_HAUTEUR-75 )
        return AJOUTCOMPAGNIE;
    if ( y < ECRAN_HAUTEUR-60 )
        return LISTENOIRE;
    if ( y < ECRAN_HAUTEUR-45 )
        return ALEATOIRE;
    if ( y < ECRAN_HAUTEUR-30 )
        return STATUS;
    if ( y < ECRAN_HAUTEUR-15 )
	return HISTORIQUE;
    return QUITTER;
}

/*** ecrire_nchar
 * Affiche n caractres de txt sur la sortie standard
 */
void ecrire_nchar(char *txt, int n)
{
    int i;
    for ( i=0 ; i < n ; i++ )
	putchar(txt[i]);
}

/*** demande_compagnie
 * Ajoute une compagnie dont l'acronyme peut tre ou non prcis  l'avance
 */
Liste_compagnie demande_compagnie(Liste_compagnie * comp, char acronyme[3])
{
    char *nom,*acro;
    int tn=0,ta=0;

    if ( acronyme == NULL ) {
	puts("Acronyme ?");
	getline(&acro,&ta,stdin);
	while ( ! acronyme_valide(acro) ) {
	    puts("Acronyme invalide, veuillez l'entrer  nouveau");
	    getline(&acro,&ta,stdin);
	}
    } else {
	acro = malloc(3);
	memcpy(acro,acronyme,3);
    }

    printf("Nom de la compagnie ");
    ecrire_nchar(acro,3);
    puts(" ?");
    getline(&nom,&tn,stdin);
    nom[strlen(nom)-1] = '\0';

    alloue_compagnie(comp,nom,acro);
    free(acro);

    return *comp;
}

/*** demande_avion
 * Ajoute un avion en crant la compagnie si besoin
 */
void demande_avion(Liste_compagnie * comp, Liste_avions *decoll,
	Liste_avions *atte, Liste_noire noire, int t)
{
    char *lecture, acro[3], heure[4], id[7], c;
    int carburant, conso, tl=0, vol;
    Liste_compagnie compagnie;

    /* Compagnie */
    puts("Compagnie ? (crivez l'acronyme)");
    getline(&lecture,&tl,stdin);
    while ( ! acronyme_valide(lecture) ) {
	puts("Acronyme invalide");
	getline(&lecture,&tl,stdin);
    }
    memcpy(acro,lecture,3);

    if ( ( compagnie = comp_connue(*comp,acro)) == NULL ) {
	puts("Compagnie inconnue, cration.");
	compagnie = demande_compagnie(comp,acro);
	if ( compagnie == NULL )
	    goto erreur;
    }

    if ( sur_liste_noire(noire,&(compagnie->comp)) ) {
	puts("Cette compagnie est sur liste noire !");
	goto erreur;
    }

    /* Cration de l'identifiant */
    vol = vollibre(compagnie->comp.avions_compagnie);
    if ( vol == -1 ) {
	puts("Compagnie pleine");
	goto erreur;
    }
    sprintf(id,"%c%c%c%03d",lecture[0],lecture[1],lecture[2],vol);

    /* Mode dcollage ou attrissage */
    puts("Cet avion va-t-il dcoller ou attrir ? (D/A)");
    do { 
	getline(&lecture,&tl,stdin);
	memcpy(&c,lecture,1);
    } while ( c != 'D' && c != 'A' );

    switch(c)
    {
	case 'D':
	    /* Heure */
	    puts("Heure de dcollage ? (format hhmm)");
	    getline(&lecture,&tl,stdin);
	    memcpy(heure,lecture,4);
	    while ( ! hvalide(lecture) || ( horaire_minute(heure) <= t+DELAI 
			&& horaire_minute(heure) >= t ) ) {
		puts("Heure invalide, entrez la  nouveau");
		getline(&lecture,&tl,stdin);
		memcpy(heure,lecture,4);
	    }
	    /* Ajout de l'avion */
	    if ( alloue_avion(&(compagnie->comp.avions_compagnie),id,0,0,heure, 
			&(compagnie->comp)) ) {
		ajouter_decollage(decoll,compagnie->comp.avions_compagnie,t);
		ecrire_nchar(id,6); puts(" ajout.");
	    } else
		goto erreur;
	    break;
	case 'A':
	    carburant = aleatoire(5,99);
	    conso = aleatoire(1,carburant%5+1);
	    /* Ajout de l'avion */
	    if ( alloue_avion(&(compagnie->comp.avions_compagnie),id,carburant,
			conso,NULL,&(compagnie->comp)) ) {
		ajouter_atterrissage(atte,compagnie->comp.avions_compagnie);
		ecrire_nchar(id,6); puts(" ajout.");
	    } else
		goto erreur;
	    break;
    }

    if ( lecture != NULL )
	free(lecture);
    return;

erreur:
    fprintf(stderr,"ERREUR : Impossible d'ajouter l'avion ");
    ecrire_nchar(id,6);
    puts(".");
    if ( lecture != NULL )
	free(lecture);
}

/*** status
 * Affiche des informations sur la compagnie passe en paramtre
 */
void status(Liste_compagnie comp, Liste_noire noire, int t)
{
    char *chaine = NULL, acro[3];
    int taille, nombreavions = 0;
    bool decollage = FAUX;
    Liste_compagnie compagnie;
    Liste_avions avions;

    puts("Acronyme de la compagnie");
    getline(&chaine,&taille,stdin);
    if ( chaine != NULL ) {
	memcpy(acro,chaine,3);
	free(chaine);
    }
    chaine = NULL;

    if ( (compagnie = comp_connue(comp,acro)) == NULL ) {
	fprintf(stderr,"Compagnie inconnue\n");
	return;
    }

    printf("Nom : %s\n",compagnie->comp.nom);

    if ( sur_liste_noire(noire,&(compagnie->comp)) ) {
	puts("Compagnie sur liste noire");
	return;
    }

    avions = compagnie->comp.avions_compagnie;

    if ( avions == NULL ) {
	puts("Aucun avions");
	return;
    }

    while ( avions != NULL ) {
	decollage = ( hvalide(avions->avion.heure_decollage) ) ? VRAI : FAUX;
	chaine = chaine_avion(avions,decollage);
	printf(chaine);
	putchar('\n');
	if ( chaine != NULL )
	    free(chaine);
	chaine = NULL;
	avions = avions->suivant_compagnie;
	nombreavions++;
    }

    printf("%d avions dans la compagnie\n",nombreavions);
}

/*** supprimer
 * Supprime un avion de la liste de dcollage
 */
void supprimer(Liste_avions * decollage, Queue * queue)
{
    char *lecture = NULL, id[6];
    int taille = 0;
    Liste_avions avion = *decollage;
    Liste_avions tmp;
    
    puts("Identifiant de l'avion  supprimer :");
    getline(&lecture,&taille,stdin);
    while ( ! identifiant_valide(lecture) ) {
	puts("Identifiant invalide. Entrez le  nouveau");
	getline(&lecture,&taille,stdin);
    }
    memcpy(id,lecture,6);
    free(lecture);

    if ( *decollage != NULL ) {
	/* Cas o le premier avion est celui  supprimer */
	if ( memcmp((*decollage)->avion.identifiant,id,6) == 0 
		&& queue->premier == NULL ) {
	    *decollage = (*decollage)->suivant_attente;
	    detruire_avion(avion);
	    ecrire_nchar(id,6);
	    puts(" supprim.");
	    return;
	} else if ( avion->suivant_attente != NULL ) {
	    /* On passe d'abord les avions qui sont en piste */
	    if ( queue->dernier != NULL )
		while ( avion != NULL && avion != queue->dernier )
		    avion = avion->suivant_attente;
	    /* Puis on tente de supprimer l'avion */
	    if ( avion != NULL ) {
		while ( avion->suivant_attente != NULL ) {
		    if ( memcmp(avion->suivant_attente->avion.identifiant,id,6)
			    == 0 ) {
			tmp = avion->suivant_attente;
			avion->suivant_attente = 
			    avion->suivant_attente->suivant_attente;
			detruire_avion(tmp);
			ecrire_nchar(id,6);
			puts(" supprim.");
			return;
		    }
		avion = avion->suivant_attente;
		}
	    }
	}
    }

    ecrire_nchar(id,6);
    puts(" inconnu ou dj sur la piste.");
}

/*** charger_fichier
 * charge un fichier de dcollages/attrissages
 */
void charger_fichier(Liste_compagnie *comp, Liste_avions *decollages, 
	Liste_avions *atterrissages, Liste_noire noire, int t, char *nomfichier)
{
    FILE *fichier;
    char *nom_fichier = NULL, *commande = NULL;
    int carbu,conso,ligne=1,taille=0;
    Liste_compagnie compagnie;
    
    if ( nomfichier == NULL ) {
	puts("Nom du fichier:");
	getline(&nom_fichier,&taille,stdin);
	nom_fichier[strlen(nom_fichier)-1] = '\0';
    } else {
	if ( (nom_fichier = malloc(strlen(nomfichier)+1)) == NULL ) {
	    fprintf(stderr,"Impossible d'ouvrir le fichier %s\n",nomfichier);
	    return;
	}
	strcpy(nom_fichier,nomfichier);
    }

    if ( (fichier = fopen(nom_fichier,"r")) == NULL ) {
	fprintf(stderr,"Impossible d'ouvrir : %s\n",nom_fichier);
	return;
    }

    free(nom_fichier);

    while ( getline(&commande,&taille,fichier) != -1 ) {
	ligne++;

	/* Commentaires */
	if ( commande[0] == '#' )
	    continue;
	
	/* On vrifie la prsence des tirets obligatoires */
	if ( commande[6] != '-' || commande[8] != '-' || commande[13] != '-'
		|| commande[16] != '-' )
	    goto syntax_error;

	if ( ! identifiant_valide(commande) )
	    goto syntax_error;

	/* Cration de la compagnie si ncessaire */
	if ( (compagnie = comp_connue(*comp,commande)) == NULL ) {
	    compagnie = demande_compagnie(comp,commande);
	    if ( compagnie == NULL ) {
		fprintf(stderr,"Cration de la compagnie impossible");
		free(commande);
		return;
	    }
	}
	
	if ( avion_connu(compagnie->comp.avions_compagnie,commande) != NULL ) {
	    ecrire_nchar(compagnie->comp.avions_compagnie->avion.identifiant,6);
	    puts(" dj existant !");
	    continue;
	}

	if ( sur_liste_noire(noire,&(compagnie->comp)) ) {
	    printf("Compagnie sur liste noire, impossible d'ajouter : ");
	    ecrire_nchar(commande,6);
	    putchar('\n');
	    continue;
	}

	/* Ajout de l'avion */
	switch(commande[7]) {
	    case 'A':
		/* Calculs de quantit de carburant et de consommation */
		carbu = 10 * (commande[14]-'0') + (commande[15]-'0');
		conso = 10 * (commande[17]-'0') + (commande[18]-'0');
		/* Chainage */
		alloue_avion(&(compagnie->comp.avions_compagnie),commande,carbu,
			conso,NULL,&(compagnie->comp));
		ajouter_atterrissage(atterrissages,
			compagnie->comp.avions_compagnie);
		break;
	    case 'D':
		if ( !hvalide(&commande[9]) )
		    goto syntax_error;
		/* Chainage */
		if ( ! hvalide(&commande[9]) )
		    goto syntax_error;
		if ( horaire_minute(&commande[9]) >= t
			&& horaire_minute(&commande[9]) <= t+DELAI )  {
		    ecrire_nchar(commande,6);
		    puts(" voulait dcoller dans un trop court dlai, dcollage annul.");
		    break;
		}
		alloue_avion(&(compagnie->comp.avions_compagnie),commande,
			0,0,&commande[9],&(compagnie->comp));
		ajouter_decollage(decollages,compagnie->comp.avions_compagnie,t);
		break;
	    default:
		goto syntax_error;
	}
    }

    printf("%d lignes lues\n",ligne);

    fclose(fichier);
    if ( commande != NULL )
	free(commande);
    return;

syntax_error:
    fprintf(stderr,"Erreur dans syntaxe dans : %s, ligne : %d\n",
	    nom_fichier,ligne);
    fprintf(stderr,"Arrt de la lecture.\n");
    fclose(fichier);
    if ( commande != NULL )
	free(commande);
}

/*** charger_compagnies
 * charge un fichier de compagnies
 * Format : Pour chaque ligne, "<acronyme sur 3 caractres> <nom>"
 */
void charger_compagnies(char *nom_fichier, Liste_compagnie * comp)
{
    FILE *fichier;
    size_t longueur = 0;
    char *ligne = NULL;
    int lignes = 0;

    if ( (fichier = fopen(nom_fichier,"r")) == NULL ) {
	fprintf(stderr,"Impossible d'ouvrir : %s\n",nom_fichier);
	return;
    }

    while ( getline(&ligne,&longueur,fichier) != -1 ) {
	lignes++;
	
	/* Commentaire */
	if ( ligne[0] == '#' )
	    continue;

	/* Lecture de compagnie */
	ligne[strlen(ligne)-1] = 0;
	if ( ligne[3] != ' ' )
	    goto syntax_error;
	if ( ! acronyme_valide(ligne) )
	    goto syntax_error;
	if ( comp_connue(*comp,ligne) != NULL ) {
	    ecrire_nchar(ligne,3);
	    puts(", compagnie existante, non ajoute.");
	    continue;
	}
	alloue_compagnie(comp,&ligne[4],ligne);
    }

    printf("%d lignes lues.\n",lignes);

    free(ligne);
    fclose(fichier);
    return;

syntax_error:
    fprintf(stderr,"Erreur de syntaxe dans %s, ligne : %d\n",
	    nom_fichier,lignes);
    fprintf(stderr,"Arrt de la lecture.\n");
    free(ligne);
    fclose(fichier);
}

/*** lire_historique
 * Affiche l'historique sur la console
 */
void lire_historique(Historique * histo)
{
    if ( histo != NULL ) {
	lire_historique(histo->suivant);
	puts(histo->txt);
    }
}

/*** mettre_en_urgence
 * Met en urgence l'avion demand en crivant URGE dans le tableau
 * heure_decollage
 */
void mettre_en_urgence(Liste_avions atterrissages)
{
    int taille = 0;
    char *ligne = NULL;

    puts("Identifiant de l'avion");
    getline(&ligne,&taille,stdin);
    while ( ! identifiant_valide(ligne) ) {
	puts("Identifiant invalide, veuillez l'entrer  nouveau");
	getline(&ligne,&taille,stdin);
    }

    while ( atterrissages != NULL ) {
	if ( memcmp(ligne,atterrissages->avion.identifiant,6) == 0 ) {
	    memcpy(atterrissages->avion.heure_decollage,"URGE",4);
	    return;
	}
	atterrissages = atterrissages->suivant_attente;
    }

    puts("Avion inconnu");
}

/*** liste_noire
 * Dtruit tous les avions au dcollage de la compagnie demande et inscrit
 * NOIR dans tous les avions en attente d'attrissage de la compagnie pour les 
 * faire attrir
 */
void liste_noire(Liste_compagnie compagnies, Liste_avions * decollages,
	Liste_avions atterrissages, Liste_noire * noire)
{
    int taille = 0;
    char *ligne = NULL;
    Liste_avions tmp;
    Liste_avions decollage;
    Liste_compagnie comp;

    puts("Acronyme de la compagnie");
    getline(&ligne,&taille,stdin);

    if ( (comp = comp_connue(compagnies,ligne)) == NULL ) {
	puts("Compagnie inconnue");
	return;
    }

    mettre_sur_liste_noire(noire,&(comp->comp));

    /* Destruction des avions au dcollage */
    while ( *decollages != NULL ) {
	if ( memcmp((*decollages)->avion.identifiant,ligne,3) == 0 ) {
	    tmp = *decollages;
	    *decollages = (*decollages)->suivant_attente;
	    detruire_avion(tmp);
	} else 
	    break;
    }
    decollage = *decollages;
    if ( decollage != NULL )
	while ( decollage->suivant_attente != NULL ) {
	    if ( memcmp(decollage->suivant_attente->avion.identifiant,ligne,3) 
		    == 0 ) {
		tmp = decollage->suivant_attente;
		decollage->suivant_attente = 
		    decollage->suivant_attente->suivant_attente;
		detruire_avion(tmp);
	    } else {
		decollage = decollage->suivant_attente;
	    }
	}

    free(ligne);
}

/*** historique
 * Ajoute l'vnement pass en argument par l'avion et l'action  l'historique
 * et inscrit l'information dans aeroport.log
 */
void historique(Historique ** historique, Action action, Liste_avions avion,
	int t)
{
    FILE *log;
    char *txt, *heure;

    if ( (log = fopen("aeroport.log","a")) == NULL )
	fprintf(stderr,"Impossible d'ouvrir aeroport.log !");

    switch(action) {
	case D:
	    txt = chaine_avion(avion,VRAI);
	    break;
	case A:
	    txt = chaine_avion(avion,FAUX);
	    break;
	case U:
	    txt = chaine_avion(avion,FAUX);
	    txt[7] = 'U';
	    break;
	case N:
	    txt = chaine_avion(avion,FAUX);
	    txt[7] = 'N';
	    break;
	case C:
	    if ( (txt = malloc(19)) == NULL ) {
		fprintf(stderr,"Impossible d'ajouter l'vnement  l'historique");
		break;
	    }
	    strncpy(txt,avion->avion.identifiant,6);
	    if ( (heure = minute_horaire(t)) == NULL ) {
		fprintf(stderr,"Impossible de convertir le temps en horaire !");
		return;
	    }
	    memcpy(&txt[9],heure,4);
	    free(heure);
	    txt[7] = 'C';
	    txt[6] = txt[8] = '-';
	    strcpy(&txt[13],"------");
	    break;
    }
    
    historique_ajouter(historique,txt);
    fprintf(log,txt);
    fprintf(log,"\n");
    fclose(log);
    if ( txt != NULL )
	free(txt);
}

