/* gestion.c
 * Fonctions de gestion de l'aroport
 */

#include "../include/aeroport.h"

/*** evenements
 * vnements automatiques ( attrissages, dcollages, etc )
 */
void evenements(Liste_compagnie * comp, Liste_avions * atterrissages, 
	Liste_avions * decollages, Queue * queue, int t,
	Historique ** historique, Liste_noire * noire, int n_pistes)
{
    /* pistes[i] vaut VRAI si la piste i est utilise */
    bool pistes[n_pistes];
    int piste = 0;
    int i;

    /* Initialisation des pistes, toutes libres */
    for ( i=0 ; i < n_pistes ; i++ )
	pistes[i] = FAUX;

    /* Dtection des urgences et mise en piste des avions en approche de
     * dcollage */
    gestion_piste(*decollages,queue,t);
    detection_urgences(*atterrissages);

    /* Utilisation des pistes */
    while ( (piste = piste_libre(pistes,n_pistes)) != -1 ) {
	pistes[piste] = atterrissage_urgence(atterrissages,historique);

	if ( pistes[piste] == FAUX )
	    pistes[piste] = decollage(decollages,queue,t,historique);

	if ( pistes[piste] == FAUX )
	    pistes[piste] = atterrissage_noire(atterrissages,historique,*noire);

	if ( pistes[piste] == FAUX )
	    if ( (pistes[piste] = atterrissage(atterrissages,historique))
		    == FAUX )
		break;
    }

    gestion_carburant(*atterrissages);
    crashs(atterrissages,historique,t);
}


/*** gestion_carburant
 * Met  jour la quantit de carburant consomme par les avions en attente
 * d'attrissage
 */
void gestion_carburant(Liste_avions atterrissages)
{
    while ( atterrissages != NULL ) {
	atterrissages->avion.carburant -= atterrissages->avion.consommation;
	atterrissages = atterrissages->suivant_attente;
    }
}

/*** crashs
 * Retire les avions qui se sont crashs de la liste d'attrissage et les
 * ajoute  l'historique
 */
void crashs(Liste_avions * atterrissages, Historique ** histo, int t)
{
    Liste_avions tmp = *atterrissages, tmp2 = NULL;

    /* On gre d'abord les crash sur la tte de liste */
    while ( *atterrissages != NULL && (*atterrissages)->avion.carburant <= 0 ) {
	historique(histo,C,*atterrissages,t);
	*atterrissages = (*atterrissages)->suivant_attente;
	detruire_avion(tmp);
	tmp = *atterrissages;
    }

    /* Avions suivants */
    if ( tmp != NULL )
	while ( tmp->suivant_attente != NULL ) {
	    if ( tmp->avion.carburant <= 0 ) {
		historique(histo,C,tmp,t);
		tmp2 = tmp->suivant_attente->suivant_attente;
		detruire_avion(tmp->suivant_attente);
		tmp->suivant_attente = tmp2;
	    } else {
		tmp = tmp->suivant_attente;
	    }
	}
}

/*** atterrissage
 * Fait attrir le prochain avion en attente
 * ( mise  jour de la liste d'attrissage + note dans l'historique )
 */
bool atterrissage(Liste_avions * atterrissages, Historique ** histo)
{
    if ( *atterrissages == NULL )
	return FAUX;

    Liste_avions atterri = *atterrissages;

    /* Note */
    historique(histo,A,atterri,0);

    /* Destruction */
    *atterrissages = (*atterrissages)->suivant_attente;
    detruire_avion(atterri);

    return VRAI;
}

/*** decollage
 * Fait dcoller un avion
 * ( mise  jour de la liste de dcollage + note dans l'historique )
 */
bool decollage(Liste_avions * decollages, Queue * queue, int t, 
	Historique ** histo)
{
    if ( queue->premier == NULL )
	return FAUX;

    Liste_avions decolle = queue->premier;

    if ( horaire_minute(decolle->avion.heure_decollage) <= t ) {
	*decollages = (*decollages)->suivant_attente;
	if ( queue->premier == queue->dernier ) {
	    queue->premier = queue->dernier = NULL;
	} else
	    queue->premier = queue->premier->suivant_attente;
	historique(histo,D,decolle,0);
	detruire_avion(decolle);
	return VRAI;
    }
    
    return FAUX;
}

/*** gestion_piste
 * Met  jour les avions en attente sur la piste
 * ( met tous les avions dont le delai d'attente est infrieur  t+DELAI 
 * et suprieur  t dans la queue )
 */
void gestion_piste(Liste_avions decollages, Queue * queue, int t)
{
    if ( decollages == NULL )
	return;

    int horaire = horaire_minute(decollages->avion.heure_decollage);

    if ( horaire <= t+DELAI && horaire >= t ) {
	if ( queue->premier == NULL ) {
	    queue->premier = decollages;
	    queue->dernier = queue->premier;
	}
	decollages = decollages->suivant_attente;
	if ( decollages == NULL ) {
	    queue->dernier = NULL;
	    return;
	}
	horaire = horaire_minute(decollages->avion.heure_decollage);
	while ( decollages != NULL && (horaire <= t+DELAI && horaire >= t) ) {
		queue->dernier = decollages;
		decollages = decollages->suivant_attente;
		horaire = horaire_minute(decollages->avion.heure_decollage);
	}
    }
}

/*** detection_urgences
 * Met les avions n'ayant plus qu'un tour possible avant crash dans la liste
 * des urgences
 */
void detection_urgences(Liste_avions atterrissages)
{
    float tours_restants;

    while ( atterrissages != NULL ) {
	tours_restants = atterrissages->avion.carburant /
	    (float) atterrissages->avion.consommation;
	/* Si l'avion ne peut faire plus qu'un seul tour avant de se crasher
	 * il est dclar comme tant en urgence */
	if ( tours_restants < 1.0 )
	    memcpy(atterrissages->avion.heure_decollage,"URGE",4);
	atterrissages = atterrissages->suivant_attente;
    }
}

/*** atterrissages_urgence
 * Fait attrir le prochain avion en urgence.
 */
bool atterrissage_urgence(Liste_avions * atterrissages, Historique ** histo)
{
    if ( *atterrissages == NULL )
	return FAUX;

    Liste_avions tmp = *atterrissages;
    Liste_avions urgent;

    /* Cas du premier avoin de la liste */
    if ( memcmp((*atterrissages)->avion.heure_decollage,"URGE",4) == 0 ) {
	urgent = *atterrissages;
	*atterrissages = (*atterrissages)->suivant_attente;

	historique(histo,U,urgent,0);
	detruire_avion(urgent);
	
	return VRAI;
    }

    /* Avions suivants */
    while ( tmp->suivant_attente != NULL ) {
	if ( memcmp(tmp->suivant_attente->avion.heure_decollage,"URGE",4) 
		== 0 ) {
	    /* Nouveau chainage */
	    urgent = tmp->suivant_attente;
	    tmp->suivant_attente = tmp->suivant_attente->suivant_attente;

	    /* Nettoyage */
	    historique(histo,U,urgent,0);
	    detruire_avion(urgent);

	    return VRAI;
	}
	tmp = tmp->suivant_attente;
    }

    return FAUX;
}

/*** atterrissages_noire
 * Fait attrir le prochain d'une compagnie sur liste noire
 */
bool atterrissage_noire(Liste_avions * atterrissages, Historique ** histo,
	Liste_noire liste)
{
    if ( *atterrissages == NULL )
	return FAUX;

    Liste_avions tmp = *atterrissages;
    Liste_avions noir;

    /* Cas du premier avion de la liste */
    if ( sur_liste_noire(liste,(*atterrissages)->avion.compagnie) ) {
	/* Nouveau chainage */
	noir = *atterrissages;
	*atterrissages = (*atterrissages)->suivant_attente;

	/* Note */
	historique(histo,N,noir,0);
	/* Nettoyage */
	detruire_avion(noir);
	
	return VRAI;
    }

    /* Avions suivants */
    while ( tmp->suivant_attente != NULL ) {
	if ( sur_liste_noire(liste,tmp->suivant_attente->avion.compagnie) ) {
	    /* Nouveau chainage */
	    noir = tmp->suivant_attente;
	    tmp->suivant_attente = tmp->suivant_attente->suivant_attente;

	    /* Note */
	    historique(histo,N,noir,0);
	    /* Nettoyage */
	    detruire_avion(noir);

	    return VRAI;
	}
	tmp = tmp->suivant_attente;
    }

    return FAUX;
}
