/* memoire.c
 * Gestion de la mmoire du programme
 */

#include "../include/aeroport.h"

/*** alloue_compagnie
 * Ajoute une compagnie
 */
bool alloue_compagnie(Liste_compagnie * comp, char * nom, char acro[3])
{
    Liste_compagnie compagnie;

    if ( (compagnie = malloc(sizeof(Cellule_compagnie))) == NULL )
	return FAUX;

    if ( (compagnie->comp.nom = malloc(strlen(nom)+1)) == NULL ) {
	free(compagnie);
	return FAUX;
    }

    /* Informations */
    compagnie->comp.avions_compagnie = NULL;
    strcpy(compagnie->comp.nom,nom);
    memcpy(compagnie->comp.acronyme,acro,3);

    /* Chainage */
    compagnie->suivant = *comp;
    *comp = compagnie;
    
    printf("Compagnie \"%s\" ajoute.\n",compagnie->comp.nom);
    return VRAI;
}

/*** alloue_avion
 * Ajoute un avion  la liste d'avions passe en paramtres
 */
bool alloue_avion(Liste_avions * avions, char id[6], int carburant, int conso,
	char heure_decollage[4], struct compagnie * compagnie)
{
    Liste_avions tmp;

    if ( (tmp = malloc(sizeof(Cellule_avion))) == NULL )
	return FAUX;

    /* Informations */
    memcpy(tmp->avion.identifiant,id,6);
    if ( heure_decollage != NULL ) {
	memcpy(tmp->avion.heure_decollage,heure_decollage,4);
	carburant = conso = 0;
    } else {
	memcpy(tmp->avion.heure_decollage,"----",4);
	tmp->avion.carburant = carburant;
	tmp->avion.consommation = conso;
    }
    tmp->avion.compagnie = compagnie;
    /* Mise  jour du lien vers la compagnie */
    if ( *avions != NULL )
	(*avions)->avion.compagnie = NULL;

    /* Chainage */
    tmp->suivant_attente = NULL;
    tmp->precedent_compagnie = NULL;
    if ( *avions != NULL )
	(*avions)->precedent_compagnie = tmp;
    tmp->suivant_compagnie = *avions;
    *avions = tmp;
    
    return VRAI;
}

/*** ajouter_decollage
 * Ajouter l'avion "avion" dans la liste de dcollage de manire classe
 * par rapport  son heure de dcollage ainsi que l'heure actuelle
 */
void ajouter_decollage(Liste_avions * decollages, Liste_avions avion, int t)
{
    if ( *decollages == NULL ) {
	*decollages = avion;
	return;
    }

    /* m1 correspond  l'heure de dcollage de "avion" en minutes depuis 0:00
     * m2 correspond  l'heure de dcollage du prochain avion  dcoller 
     * m3 correspond  l'heure de dcollage du suivant du prochain */
    int m1 = horaire_minute(avion->avion.heure_decollage),
	m2 = horaire_minute((*decollages)->avion.heure_decollage),
	m3;
    Liste_avions tmp;

    /* L'avion va devenir le premier  dcoller */
    if ( m1 < m2 && m1 > t+5 ) {
	avion->suivant_attente = *decollages;
	*decollages = avion;
    } else {
	tmp = *decollages;
	while ( tmp->suivant_attente != NULL ) {
	    m2 = horaire_minute(tmp->suivant_attente->avion.heure_decollage);
	    if ( tmp->suivant_attente->suivant_attente != NULL )
		m3 = horaire_minute(tmp->suivant_attente->suivant_attente->
			avion.heure_decollage);
	    else
		m3 = -1;
	    if ( (m1 <= m2 && m1 > t+5 && m2 > t+5 ) 
		    || (m1 <= m2 && m2 < t && m1 < t) ) {
		avion->suivant_attente = tmp->suivant_attente;
		tmp->suivant_attente = avion;
		return;
	    } else if (m3 != -1 && m1 >= m2 && m1 >= m3 && m3 < t && m2 > t ) {
		avion->suivant_attente = tmp->suivant_attente->suivant_attente;
		tmp->suivant_attente->suivant_attente = avion;
		return;
	    }
	    tmp = tmp->suivant_attente;
	}
	tmp->suivant_attente = avion;
    }
}

/*** ajouter_atterrissage
 * Ajoute l'avion "avion" dans la liste des atterrissages de manire classe
 * par rapport au nombre de tours qu'il peut faire en l'air
 */
void ajouter_atterrissage(Liste_avions * atterrissages, Liste_avions avion)
{
    if ( *atterrissages == NULL ) {
	*atterrissages = avion;
	return;
    }

    /* t1 correspond au nombre de tours que peut faire l'avion "avion"
     * t2 correspond au nombre de tours que peut faire l'avion "slectionn" 
     * l'attrissage (tmp) */
    float t1 = avion->avion.carburant / (float) avion->avion.consommation,
	  t2 = (*atterrissages)->avion.carburant / 
	      (float) (*atterrissages)->avion.consommation;
    Liste_avions tmp;

    if ( t1 < t2 ) {
	avion->suivant_attente = *atterrissages;
	*atterrissages = avion;
    } else {
	tmp = *atterrissages;
	while ( tmp->suivant_attente != NULL ) {
	    t2 = tmp->suivant_attente->avion.carburant 
		/ (float) tmp->suivant_attente->avion.consommation;
	    if ( t1 < t2 ) {
		avion->suivant_attente = tmp->suivant_attente;
		tmp->suivant_attente = avion;
		return;
	    }
	    tmp = tmp->suivant_attente;
	}
	tmp->suivant_attente = avion;
    }
}

/*** historique_ajouter
 * Ajoute un vnement  l'historique
 */
bool historique_ajouter(Historique ** historique, char *txt)
{
    Historique * tmp;

    if ( (tmp = malloc(sizeof(Historique))) == NULL )
	return FAUX;

    strcpy(tmp->txt,txt);
    tmp->suivant = *historique;
    *historique = tmp;
    
    return VRAI;
}

/*** mettre_sur_liste_noire
 * Ajoute un pointeur sur une compagnie  la liste noire
 */
bool mettre_sur_liste_noire(Liste_noire * noire, Compagnie * comp)
{
    Liste_noire tmp;

    if ( (tmp = malloc(sizeof(Cellule_noire))) == NULL ) {
	fprintf(stderr,"Impossible d'ajouter  la liste noire");
	return FAUX;
    }

    tmp->comp = comp;
    tmp->suiv = *noire;
    *noire = tmp;

    return VRAI;
}


/*** detruire_avion
 * Libre la mmoire de l'avion passe en argument en mettant  jour la
 * liste d'avions avant.
 */
void detruire_avion(Liste_avions avions)
{
    /* Mise  jour du lien vers la compagnie */
    if ( avions->precedent_compagnie == NULL
	    && avions->suivant_compagnie != NULL ) {
	avions->avion.compagnie->avions_compagnie = avions->suivant_compagnie;
	avions->suivant_compagnie->avion.compagnie =
	    avions->avion.compagnie;
    }
    
    /* Cas du dernier avion de la compagnie */
    if ( avions->suivant_compagnie == NULL
	    && avions->precedent_compagnie == NULL )
	avions->avion.compagnie->avions_compagnie = NULL;

    /* Suppression */
    if ( avions->suivant_compagnie != NULL )
	avions->suivant_compagnie->precedent_compagnie =
	    avions->precedent_compagnie;
    if ( avions->precedent_compagnie != NULL )
	avions->precedent_compagnie->suivant_compagnie =
	    avions->suivant_compagnie;
    free(avions);
}

/*** liberer_avions
 * Libre la mmoire de tous les avions de la liste passe en arguments
 */
void liberer_avions(Liste_avions * avions)
{
    Liste_avions tmp;

    while ( *avions != NULL ) {
	tmp = (*avions)->suivant_compagnie;
	detruire_avion(*avions);
	*avions = tmp;
    }
}

/*** liberer_compagnies
 * Libre la mmoire de toutes les compagnies ainsi que de leurs avions
 */
void liberer_compagnies(Liste_compagnie * comp)
{
    Liste_compagnie tmp;

    while ( *comp != NULL ) {
	tmp = (*comp)->suivant;
	liberer_avions(&((*comp)->comp.avions_compagnie));
	free((*comp)->comp.nom);
	free(*comp);
	*comp = tmp;
    }
}

/*** effacer_historique
 * Efface tout l'historique
 */
void effacer_historique(Historique ** historique)
{
    Historique * tmp = *historique;

    while ( tmp != NULL ) {
	tmp = (*historique)->suivant;
	free(*historique);
	*historique = tmp;
    }
}

/*** effacer_liste_noire
 * Efface toute la liste noire
 */
void effacer_liste_noire(Liste_noire * noire)
{
    Liste_noire tmp;

    while ( *noire != NULL ) {
	tmp = (*noire)->suiv;
	free(*noire);
	*noire = tmp;
    }
}
